// $Id: bubbletimer.js,v 1.4.2.6 2009/05/17 11:07:12 itarato Exp $


/**
 * @file
 * Bubbletimer main javascript
 */


String.prototype.log = function() {
  if (typeof(console) == 'object') {
    console.log("log -- " + this.toString());
  }
}


// How much time the stat update should be delayed
var BT_UPDATE_DELAY_MS = 3000;
var BT_SECONDS_INTERVAL = 15 * 60; // 15 minutes
//SHIFT - for interval selection
var BT_INTERVAL_KEY_CODE = 16;
//CTRL - for multijob selections
var BT_MULTI_KEY_CODE = 17;
var BT_CHECKBOX_SELECTOR = 'table.bt_checkbox div input:checkbox';

var BT_HIGHLIGHT_CLASS = 'bt_title_highlight';

// Finding a checkbox which timestamp its related to
var bt_job_timestamp_regexp = new RegExp(/^bt_job_(\d*)_timestamp_(\d*)$/g);
// Check if keys are down
var interval_key_is_active = false;
var multi_key_is_active = false;
// setTimeout var
var bt_update_delay;
// true if changes occured on the form
var bt_update_need_save_flag = false;
// nodes
var bt_job_array = [];
// true if charts isnt refreshed
var bt_first_reload = true;
// last selected slot
var bubbletimer_last_selected = false;

var bubbletimer_history_backward_array = [];
var bubbletimer_history_forward_array  = [];

var bt_checkbox_image_on;
var bt_checkbox_image_off;

var BUBBLETIMER_SLOT_UNSELECT = -1;
var BUBBLETIMER_SLOT_DEFAULT_SELECT = 0;
var BUBBLETIMER_SLOT_SELECT   = 1;
var BUBBLETIMER_NO_HISTORY_SAVE = 1;

// Table slider info
// For boosting, hiding comes from php: @see bubbletimer.page.inc at line 320,358
var BUBBLETIMER_TABLE_VISIBLE_HOURS = 10;
var bt_table_first_visible_hour = 8;
var bt_table_step_value = 1;

// Cache for highlighting
var bt_cache_highlighted_title = null;
var bt_cache_highlighted_hour = null;

if (Drupal.jsEnabled) {
  $(function() {
    "init()".log();

    bt_checkbox_image_on  = bt_image_path + '/cb_on.gif';
    bt_checkbox_image_off = bt_image_path + '/cb_off.gif';
    
    // During page reload browser restore all hidden values but image states
    bt_restore_selection();

    // Main BT table init
    bt_init_table();
    
    if (bt_piechart_enable || bt_barchart_enable) {
      $('#bt_chart_title').show();
    }

    // Node type selector
    $('#edit-bt-node-type').bind('change', _bubbletimer_node_select_refresh).trigger('change');
    
    // Add actions for date stepper
    $('input#bt_date_prev').click(function(){
      $('select#edit-bt-date-year').val(bubbletimer_date_stepper.previous.year);
      $('select#edit-bt-date-month').val(bubbletimer_date_stepper.previous.month);     
      $('select#edit-bt-date-day').val(bubbletimer_date_stepper.previous.day);
      $('form#bubbletimer-date-selector-form').submit();
    });
    $('input#bt_date_next').click(function(){
      $('select#edit-bt-date-year').val(bubbletimer_date_stepper.next.year);
      $('select#edit-bt-date-month').val(bubbletimer_date_stepper.next.month);     
      $('select#edit-bt-date-day').val(bubbletimer_date_stepper.next.day);
      $('form#bubbletimer-date-selector-form').submit();
    });
    $('input#bt_date_now').click(function(){
      $('select#edit-bt-date-year').val(bubbletimer_date_stepper.today.year);
      $('select#edit-bt-date-month').val(bubbletimer_date_stepper.today.month);
      $('select#edit-bt-date-day').val(bubbletimer_date_stepper.today.day);
      $('form#bubbletimer-date-selector-form').submit();
    });
    
    // Attach key event listeners
    $(window).keydown(function(event){
      switch (event.keyCode) {
        case BT_INTERVAL_KEY_CODE:
          interval_key_is_active = true;
          multi_key_is_active = false;
          break;
        case BT_MULTI_KEY_CODE:
          multi_key_is_active = true;
          interval_key_is_active = false;
          break;
        default:
          interval_key_is_active = false;
          multi_key_is_active = false;
          break;
      }
    });
    $(window).keyup(function(event){
      interval_key_is_active = false;
      multi_key_is_active = false;
      bt_multiselect_fst_item = null;
    });

    // Cache node ids
    $.each($('a.job_node'), function() {
      bt_job_array.push($(this).attr('nid'));
    });

    // Update stats
    bt_update_stats();
  });
}
	

/**
 * Handle a checkbox click event
 * If CTRL is down, multiple selection allowed, otherwise doesn't
 * @return
 */
function bubbletimer_slot_toggle(oImg) {
  slot_img = $(oImg);

  // Setup a notifier to make us save the changes
  if (!bt_update_need_save_flag) {
    bt_update_need_save_flag = true;
    $('div#bt_notify_to_save').show();
  }
  
  // Multi task selection
  if (multi_key_is_active) {
    _bubbletimer_slot_select.apply(slot_img);
    return;
  }  
  
  var slot_wrapper = slot_img.parent();
  var hidden = $(':hidden', slot_wrapper);
  
  // Interval selection
  if (interval_key_is_active) {
    if (bubbletimer_last_selected) {
      var last_hidden = $(':hidden', bubbletimer_last_selected.parent());
      // Finish a multiselection
      var nid = hidden.attr('nid');
      // Only in the same line
      if (last_hidden.attr('nid') == nid) {
        var timestamp_from = Math.min(hidden.attr('timestamp'), last_hidden.attr('timestamp'));
        var timestamp_to   = Math.max(hidden.attr('timestamp'), last_hidden.attr('timestamp'));
        _bubbletimer_select_range(timestamp_from, timestamp_to, nid);
        bt_update_stats();
      }
    }
    return;
  }
  
  // Need manual help to unselect a slot
  if (hidden.val() == 1) {
    _bubbletimer_slot_select.apply(slot_img);
    bt_update_stats();
    return;
  }
  
  // Only 1 checked slot in one column
  var timestamp = hidden.attr('timestamp');
  $.each($('img', $("table#bt_table input:hidden[@value='1'][@timestamp='" + timestamp + "']").parent()), function() {
    _bubbletimer_slot_select.apply(this, [BUBBLETIMER_SLOT_UNSELECT]);
  });
  
  // Click event
  _bubbletimer_slot_select.apply(slot_img);
  // Update stat
  bt_update_stats();
}


/**
 * Loader icon on
 * @return
 */
function bt_throbber_on() {
  $('span.bt_chart_updater').addClass('occupied');
}


/**
 * Loader icon off
 * @return
 */
function bt_throbber_off() {
  $('span.bt_chart_updater').removeClass('occupied');
}


/**
 * @PRIVATE
 * 
 * Proxy event for clicking on a slot
 * The image state has to be syncronized with the checkbox's state
 * 
 * @param forced_select
 *  if BUBBLETIMER_SLOT_SELECT then the result will be a selected slot
 *  if BUBBLETIMER_SLOT_UNSELECT then the result will be an unselected slot
 * @param _history_no_save
 *  if BUBBLETIMER_NO_HISTORY_SAVE then the event won't be saved to the history
 *  (it's necessary in a range selection, we want to save range in one peace)
 */
function _bubbletimer_slot_select(forced_select, _history_no_save) {
  var do_select       = forced_select     === BUBBLETIMER_SLOT_SELECT;
  var do_unselect     = forced_select     === BUBBLETIMER_SLOT_UNSELECT;
  var history_no_save = _history_no_save  === BUBBLETIMER_NO_HISTORY_SAVE;
  var slot_img = $(this);
  var hidden = $(':hidden', slot_img.parent());
  if ((slot_img.attr('src') == bt_checkbox_image_on && !do_select) || do_unselect) {
    slot_img.attr('src', bt_checkbox_image_off);
    hidden.val(0);
  } else {
    slot_img.attr('src', bt_checkbox_image_on);
    hidden.val(1);
    bubbletimer_last_selected = slot_img;
  }
  
  if (!history_no_save) {
    _bubbletimer_save_history(slot_img);
  }
}


/**
 * @PRIVATE
 * 
 * Select a range of slots
 * 
 * @param timestamp_from
 * @param timestamp_to
 * @param nid
 */
function _bubbletimer_select_range(timestamp_from, timestamp_to, nid) {
  ("range " + timestamp_from + ".." + timestamp_to).log();
  var history = [];
  for (var t = timestamp_from; t <= timestamp_to; t += BT_SECONDS_INTERVAL) {
    var slot_img = $('img', $("table#bt_table input:hidden[timestamp='" + t + "'][nid='" + nid + "']").parent());
    _bubbletimer_slot_select.apply(slot_img, [BUBBLETIMER_SLOT_SELECT, BUBBLETIMER_NO_HISTORY_SAVE]);
    history.push(slot_img);
  }
  bubbletimer_history_backward_array.pop();
  _bubbletimer_save_history(history);
}


/**
 * @PRIVATE
 * 
 * Swapping note-type selectors on the [add node] from
 * Nodes should be displayed only from the selected node-type
 */
function _bubbletimer_node_select_refresh() {
  $('.bubbletimer_node_select').parent().css('display', 'none');
  $('select[@node_type="' + $('#edit-bt-node-type').val() + '"]').parent().show();
}


function bt_restore_selection() {
  $.each($('img', $("table#bt_table input:hidden[value='1']").parent()), function() {
    _bubbletimer_slot_select.apply(this, [BUBBLETIMER_SLOT_SELECT]);
  });
}


function bt_init_table() {
  "hide columns".log();
  // For boosting, hiding comes from php: @see bubbletimer.page.inc at line 320,358
  //bt_table_visibility_refresh();
  $('#bt_table_step_left').bind('click', bt_table_step_left);
  $('#bt_table_step_right').bind('click', bt_table_step_right);

  // Setup title/hour highlighting
  $('.bt_slot_td').mouseover(function() {
    $(bt_cache_highlighted_title).removeClass('highlight');
    $(bt_cache_highlighted_hour).removeClass('highlight');

    $("td.bt_title_td[nid='" + $(this).attr('nid') + "']").addClass('highlight');
    $("td.bt_hour_td[hour='" + $(this).attr('hour') + "']").addClass('highlight');
    
    bt_cache_highlighted_title = $("td.bt_title_td[nid='" + $(this).attr('nid') + "']");
    bt_cache_highlighted_hour  = $("td.bt_hour_td[hour='" + $(this).attr('hour') + "']");
  });
  $('.bt_slot_td').mouseout(function() {
    $(bt_cache_highlighted_title).removeClass('highlight');
    $(bt_cache_highlighted_hour).removeClass('highlight');
  });
}


function bt_table_visibility_refresh() {
  for (var i = 0; i < 24; i++) {
    if (i < bt_table_first_visible_hour || i >= bt_table_first_visible_hour + BUBBLETIMER_TABLE_VISIBLE_HOURS) {
      $("table#bt_table td[hour='" + i + "']").hide();
    } else {
      $("table#bt_table td[hour='" + i + "']").show();
    }
  }
}


function bt_table_step_left() {
  $('#bt_table_step_right').show();
  var old_bt_table_first_visible_hour = bt_table_first_visible_hour;
  bt_table_first_visible_hour -= bt_table_step_value;
  if (bt_table_first_visible_hour < 0) {
    bt_table_first_visible_hour = 0;
  }
  if (bt_table_first_visible_hour == 0) {
    $('#bt_table_step_left').hide();
  }
  for (var i = bt_table_first_visible_hour; i < old_bt_table_first_visible_hour; i++) {
    $("table#bt_table td[hour='" + i + "']").show();
    $("table#bt_table td[hour='" + (i + BUBBLETIMER_TABLE_VISIBLE_HOURS) + "']").hide();
  }
}


function bt_table_step_right() {
  $('#bt_table_step_left').show();
  var old_bt_table_first_visible_hour = bt_table_first_visible_hour;
  bt_table_first_visible_hour += bt_table_step_value;
  if (bt_table_first_visible_hour > 24 - BUBBLETIMER_TABLE_VISIBLE_HOURS) {
    bt_table_first_visible_hour = 24 - BUBBLETIMER_TABLE_VISIBLE_HOURS;
  }
  if (bt_table_first_visible_hour == 24 - BUBBLETIMER_TABLE_VISIBLE_HOURS) {
    $('#bt_table_step_right').hide();
  }
  for (var i = old_bt_table_first_visible_hour; i < bt_table_first_visible_hour; i++) {
    $("table#bt_table td[hour='" + i + "']").hide();
    $("table#bt_table td[hour='" + (i + BUBBLETIMER_TABLE_VISIBLE_HOURS) + "']").show();
  }
}